import { Inject, InjectionToken, NgModule } from '@angular/core';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { Observable as Observable$1 } from 'rxjs/Observable';
import { Observable as Observable$2 } from 'rxjs/Rx';

var FRIENDLY_ERROR_MESSAGES$1 = {
    MISSING_OBSERVABLE_ITEM: "is missing from the layer misspelled name ? as soon as you provide correct name value will be emitted!"
};
var CacheLayer = /** @class */ (function () {
    /**
     * @param {?} layer
     */
    function CacheLayer(layer) {
        var _this = this;
        this.items = new BehaviorSubject$1([]);
        this.map = new Map();
        this.name = layer.name;
        this.config = layer.config;
        if (this.config.localStorage) {
            layer.items.forEach(function (item) { return _this.map.set(item['key'], item); });
            if (layer.items.constructor === BehaviorSubject$1) {
                layer.items = layer.items.getValue() || [];
            }
            this.items.next(this.items.getValue().concat(layer.items));
        }
        this.initHook(layer);
    }
    /**
     * @param {?} config
     * @return {?}
     */
    CacheLayer.createCacheParams = function (config) {
        if (config.params.constructor === Object) {
            return; // Todo
        }
        else if (config.constructor === String) {
            return; // Todo
        }
        else if (config.params.constructor === Number) {
            return; // Todo
        }
        else if (config.params.constructor === Array) {
            return; // Todo
        }
    };
    /**
     * @param {?} name
     * @return {?}
     */
    CacheLayer.prototype.get = function (name) {
        return this.map.get(name);
    };
    /**
     * @param {?} layer
     * @return {?}
     */
    CacheLayer.prototype.initHook = function (layer) {
        if (this.config.maxAge) {
            this.onExpireAll(layer);
        }
    };
    /**
     * @param {?} layer
     * @return {?}
     */
    CacheLayer.prototype.onExpireAll = function (layer) {
        var _this = this;
        layer.items.forEach(function (item) { return _this.onExpire(item['key']); });
    };
    /**
     * @param {?} layerItem
     * @return {?}
     */
    CacheLayer.prototype.putItemHook = function (layerItem) {
        if (this.config.maxAge) {
            this.onExpire(layerItem['key']);
        }
    };
    /**
     * @param {?} key
     * @return {?}
     */
    CacheLayer.prototype.getItem = function (key) {
        if (this.map.has(key)) {
            return this.get(key);
        }
        else {
            return null;
        }
    };
    /**
     * @param {?} layerItem
     * @return {?}
     */
    CacheLayer.prototype.putItem = function (layerItem) {
        this.map.set(layerItem['key'], layerItem);
        var /** @type {?} */ item = this.get(layerItem['key']);
        var /** @type {?} */ filteredItems = this.items.getValue().filter(function (item) { return item['key'] !== layerItem['key']; });
        if (this.config.localStorage) {
            localStorage.setItem(this.name, JSON.stringify(/** @type {?} */ ({
                config: this.config,
                name: this.name,
                items: filteredItems.concat([item])
            })));
        }
        this.items.next(filteredItems.concat([item]));
        this.putItemHook(layerItem);
        return layerItem;
    };
    /**
     * @param {?} key
     * @return {?}
     */
    CacheLayer.prototype.onExpire = function (key) {
        var _this = this;
        Observable$2
            .create(function (observer) { return observer.next(); })
            .timeoutWith(this.config.maxAge, Observable$2.of(1))
            .skip(1)
            .subscribe(function (observer) { return _this.removeItem(key); });
    };
    /**
     * @param {?} key
     * @return {?}
     */
    CacheLayer.prototype.removeItem = function (key) {
        var /** @type {?} */ newLayerItems = this.items.getValue().filter(function (item) { return item['key'] !== key; });
        if (this.config.localStorage) {
            var /** @type {?} */ newLayer = ({
                config: this.config,
                name: this.name,
                items: newLayerItems
            });
            localStorage.setItem(this.name, JSON.stringify(newLayer));
        }
        this.map.delete(key);
        this.items.next(newLayerItems);
    };
    /**
     * @param {?} key
     * @return {?}
     */
    CacheLayer.prototype.getItemObservable = function (key) {
        var _this = this;
        this.map.has(key) ? null : console.error("Key: " + key + " " + FRIENDLY_ERROR_MESSAGES$1.MISSING_OBSERVABLE_ITEM);
        return this.items.asObservable().filter(function () { return _this.map.has(key); }).map(function (res) { return res[0]; });
    };
    /**
     * @return {?}
     */
    CacheLayer.prototype.flushCache = function () {
        var _this = this;
        return this.items.asObservable()
            .map(function (items) {
            items.forEach(function (i) { return _this.removeItem(i['key']); });
            return true;
        });
    };
    return CacheLayer;
}());

// console.log(Array.from(this.keys()))

var CacheServiceConfigInterface = /** @class */ (function () {
    function CacheServiceConfigInterface() {
        this.deleteOnExpire = 'aggressive';
        this.cacheFlushInterval = 60 * 60 * 1000;
        this.maxAge = 15 * 60 * 1000;
        this.localStorage = false;
    }
    return CacheServiceConfigInterface;
}());

var INTERNAL_PROCEDURE_CACHE_NAME = 'cache_layers';
var FRIENDLY_ERROR_MESSAGES = {
    TRY_TO_UNSUBSCRIBE: 'Someone try to unsubscribe from collection directly... agghhh.. read docs! Blame: ',
    LOCAL_STORAGE_DISABLED: 'LocalStorage is disabled switching to regular in-memory storage.Please relate issue if you think it is enabled and there is a problem with the library itself.'
};
var CacheService = /** @class */ (function () {
    /**
     * @param {?} config
     */
    function CacheService(config) {
        var _this = this;
        this.config = config;
        this._cachedLayers = new BehaviorSubject$1([]);
        this.map = new Map();
        if (this.config.localStorage && CacheService.isLocalStorageUsable()) {
            var layers = JSON.parse(localStorage.getItem(INTERNAL_PROCEDURE_CACHE_NAME));
            if (layers) {
                layers.forEach(function (layer) {
                    var cachedLayer = JSON.parse(localStorage.getItem(layer));
                    if (cachedLayer) {
                        _this.createLayer(cachedLayer);
                    }
                });
            }
            else {
                localStorage.setItem(INTERNAL_PROCEDURE_CACHE_NAME, JSON.stringify([]));
            }
        }
    }
    /**
     * @template T
     * @param {?} cacheLayer
     * @return {?}
     */
    CacheService.createCacheInstance = function (cacheLayer) {
        return new CacheLayer(cacheLayer);
    };
    /**
     * @return {?}
     */
    CacheService.isLocalStorageUsable = function () {
        var /** @type {?} */ error = [];
        try {
            localStorage.setItem('test-key', JSON.stringify({ key: 'test-object' }));
            localStorage.removeItem('test-key');
        }
        catch (e) {
            error.push(e);
            console.log(FRIENDLY_ERROR_MESSAGES.LOCAL_STORAGE_DISABLED);
        }
        return error.length ? false : true;
    };
    /**
     * @template T
     * @param {?} name
     * @return {?}
     */
    CacheService.prototype.getLayer = function (name) {
        var /** @type {?} */ exists = this.map.has(name);
        if (!exists) {
            return this.createLayer({ name: name });
        }
        return this.map.get(name);
    };
    /**
     * @template T
     * @param {?} layer
     * @return {?}
     */
    CacheService.prototype.createLayer = function (layer) {
        var /** @type {?} */ exists = this.map.has(layer.name);
        if (exists) {
            return this.map.get(layer.name);
        }
        layer.items = layer.items || [];
        layer.config = layer.config || this.config || CACHE_MODULE_DI_CONFIG;
        var /** @type {?} */ cacheLayer = CacheService.createCacheInstance(layer);
        if (layer.config.localStorage && CacheService.isLocalStorageUsable()) {
            localStorage.setItem(INTERNAL_PROCEDURE_CACHE_NAME, JSON.stringify(CacheService.getLayersFromLS().filter(function (l) { return l !== cacheLayer.name; }).concat([cacheLayer.name])));
            localStorage.setItem(cacheLayer.name, JSON.stringify(layer));
        }
        this.map.set(cacheLayer.name, cacheLayer);
        this._cachedLayers.next(this._cachedLayers.getValue().concat([cacheLayer]));
        this.LayerHook(cacheLayer);
        return cacheLayer;
    };
    /**
     * @template T
     * @param {?} layerInstance
     * @return {?}
     */
    CacheService.prototype.LayerHook = function (layerInstance) {
        this.protectLayerFromInvaders(layerInstance);
        debugger;
        if (layerInstance.config.cacheFlushInterval || this.config.cacheFlushInterval) {
            this.OnExpire(layerInstance);
        }
    };
    /**
     * @template T
     * @param {?} cacheLayer
     * @return {?}
     */
    CacheService.prototype.protectLayerFromInvaders = function (cacheLayer) {
        cacheLayer.items.constructor.prototype.unsubsribeFromLayer = cacheLayer.items.constructor.prototype.unsubscribe;
        cacheLayer.items.constructor.prototype.unsubscribe = function () {
            console.error(FRIENDLY_ERROR_MESSAGES.TRY_TO_UNSUBSCRIBE + cacheLayer.name);
        };
    };
    /**
     * @template T
     * @param {?} layerInstance
     * @return {?}
     */
    CacheService.prototype.OnExpire = function (layerInstance) {
        var _this = this;
        Observable$1
            .create(function (observer) { return observer.next(); })
            .timeoutWith(layerInstance.config.cacheFlushInterval || this.config.cacheFlushInterval, Observable$1.of(1))
            .skip(1)
            .subscribe(function (observer) { return _this.removeLayer(layerInstance); });
    };
    /**
     * @template T
     * @param {?} layerInstance
     * @return {?}
     */
    CacheService.prototype.removeLayer = function (layerInstance) {
        this.map.delete(layerInstance.name);
        if (this.config.localStorage) {
            localStorage.removeItem(layerInstance.name);
            localStorage.setItem(INTERNAL_PROCEDURE_CACHE_NAME, JSON.stringify(CacheService.getLayersFromLS().filter(function (layer) { return layer !== layerInstance.name; })));
        }
        this._cachedLayers.next(this._cachedLayers.getValue().filter(function (layer) { return layer.name !== layerInstance.name; }).slice());
    };
    /**
     * @param {?} name
     * @param {?} newCacheLayers
     * @return {?}
     */
    CacheService.prototype.transferItems = function (name, newCacheLayers) {
        var _this = this;
        var /** @type {?} */ oldLayer = this.getLayer(name);
        var /** @type {?} */ newLayers = [];
        newCacheLayers.forEach(function (layerName) {
            var /** @type {?} */ newLayer = _this.createLayer(layerName);
            oldLayer.items.getValue().forEach(function (item) { return newLayer.putItem(item); });
            newLayers.push(newLayer);
        });
        return newLayers;
    };
    /**
     * @return {?}
     */
    CacheService.getLayersFromLS = function () {
        return JSON.parse(localStorage.getItem(INTERNAL_PROCEDURE_CACHE_NAME));
    };
    /**
     * @param {?=} force
     * @return {?}
     */
    CacheService.prototype.flushCache = function (force) {
        var _this = this;
        var /** @type {?} */ oldLayersNames;
        return this._cachedLayers.take(1)
            .map(function (layers) {
            oldLayersNames = layers.map(function (l) { return l.name; });
            layers.forEach(function (layer) { return _this.removeLayer(layer); });
            if (force) {
                localStorage.removeItem(INTERNAL_PROCEDURE_CACHE_NAME);
            }
            else {
                oldLayersNames.forEach(function (l) { return _this.createLayer({ name: l }); });
            }
            return true;
        });
    };
    /**
     * @nocollapse
     */
    CacheService.ctorParameters = function () { return [
        { type: CacheServiceConfigInterface, decorators: [{ type: Inject, args: [CACHE_MODULE_CONFIG,] },] },
    ]; };
    return CacheService;
}());

var CACHE_MODULE_CONFIG = new InjectionToken('module.config');
var CACHE_MODULE_DI_CONFIG = ({
    deleteOnExpire: 'aggressive',
    cacheFlushInterval: 60 * 60 * 1000,
    maxAge: 15 * 60 * 1000
});
var CacheModule = /** @class */ (function () {
    function CacheModule() {
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    CacheModule.forRoot = function (config) {
        return {
            ngModule: CacheModule,
            providers: [
                { provide: CACHE_MODULE_CONFIG, useValue: config || CACHE_MODULE_DI_CONFIG },
                CacheService
            ]
        };
    };
    CacheModule.decorators = [
        { type: NgModule },
    ];
    /**
     * @nocollapse
     */
    CacheModule.ctorParameters = function () { return []; };
    return CacheModule;
}());

export { CACHE_MODULE_CONFIG, CACHE_MODULE_DI_CONFIG, CacheModule, CacheService, CacheLayer, CacheServiceConfigInterface };
